﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;

namespace HeuristicLab.DataPreprocessing.Implementations
{
    class PreprocessingDataManipulation
    {
        private IPreprocessingData preprocessingData;
        private StatisticInfo statisticInfo;

        public PreprocessingDataManipulation(IPreprocessingData _prepocessingData) {
            preprocessingData = _prepocessingData;
            statisticInfo = new StatisticInfo(preprocessingData);
        }

        public void ReplaceIndicesByValue<T>(string variableName, IEnumerable<int> indices, T value)
        {
            foreach (int index in indices)
            {
                preprocessingData.SetCell<T>(variableName, index, value);
            }
        }

        public void ReplaceIndicesByAverageValue(string variableName, IEnumerable<int> indices)
        {
            double average = statisticInfo.GetAverage(variableName);
            ReplaceIndicesByValue<double>(variableName, indices, average);
        }

        public void ReplaceIndicesByMedianValue(string variableName, IEnumerable<int> indices)
        {
            double median = statisticInfo.GetMedian(variableName);
            ReplaceIndicesByValue<double>(variableName, indices, median);
        }

        public void ReplaceIndicesByRandomValue(string variableName, IEnumerable<int> indices)
        {
            Random r = new Random();

            double max = statisticInfo.GetMax<double>(variableName);
            double min = statisticInfo.GetMin<double>(variableName);
            double randMultiplier = (max - min);
            foreach (int index in indices)
            {
                double rand = r.NextDouble() * randMultiplier + min;
                preprocessingData.SetCell<double>(variableName, index, rand);
            }
        }

        public void ReplaceIndicesByLinearInterpolationOfNeighbours(string variableName, IEnumerable<int> indices)
        {
            int countValues = preprocessingData.GetValues<double>(variableName).Count();
            foreach (int index in indices)
            {
                // dont replace first or last values
                if (index > 0 && index < countValues)
                {
                    double prev = preprocessingData.GetCell<double>(variableName, index - 1);
                    double next = preprocessingData.GetCell<double>(variableName, index + 1);

                    double interpolated = (prev + next) / 2;

                    preprocessingData.SetCell<double>(variableName, index, interpolated);
                }
            }
        }

        public void ReplaceIndicesByMostCommonValue(string variableName, IEnumerable<int> indices)
        {
            if (preprocessingData.IsType<double>(variableName))
            {
                ReplaceIndicesByValue<double>(variableName, indices, statisticInfo.GetMostCommonValue<double>(variableName));
            }
            else if (preprocessingData.IsType<string>(variableName))
            {
                ReplaceIndicesByValue<string>(variableName, indices, statisticInfo.GetMostCommonValue<string>(variableName));
            }
            else if (preprocessingData.IsType<DateTime>(variableName))
            {
                ReplaceIndicesByValue<DateTime>(variableName, indices, statisticInfo.GetMostCommonValue<DateTime>(variableName));
            }
            else
            {
                throw new ArgumentException("column with index: " + variableName + " contains a non supported type.");
            }
        }
    }
}
